/**
 * RO_Monk.cpp
 * Lum, Darrell
 * CPE 471-01
 * Final Project, due 6 June 2006
 * Z. Wood
 * 
 * A monk is a character class in the MMORPG Ragnarok Online.  It can be drawn
 * in the world as a human figure with assorted headgear, spirit spheres, and
 * fury aura.  Certain skill animations are also activated when the keyboard
 * is pressed (the keyboard function is in main_release.cpp).
 *
 * Original Source: N/A
 * Edits:
 * 1. DAL:  implemented final project.
 **/

#include "include.h"

const int __DEBUG = 1; // Debugging flag for output

/**
 * Construct a monk and load its wireframe model from file. Commented 
 * readFile() calls either indicated a depreciated body part or two body parts
 * that are loaded from one file and then mirrored (e.g. monk_rfoot.m and
 * monk_lfoot.m).
 */
RO_Monk::RO_Monk() : numSpheres(0), furyState(false)
{
    // Load the Guillotine Fist character texture
    gfist = new Texture("gfist.bmp", 1); 

    // Generate a Guillotine Fist strike blast
    wave = new GF_Wave[30];

    // Load the monk's wireframe model
    readFile(this, "monk_hat.m", points, triangles, RO_UPPER_HEADGEAR);
    //readFile(this, "monk_hair.m", points, triangles, RO_MIDDLE_HEADGEAR);
    readFile(this, "monk_cape.m", points, triangles, RO_LOWER_HEADGEAR);
    readFile(this, "monk_torso.m", points, triangles, RO_TORSO);
    readFile(this, "monk_head.m", points, triangles, RO_HEAD);
    readFile(this, "monk_rarm.m", points, triangles, RO_RIGHT_ARM);
    //readFile(this, "monk_larm.m", points, triangles, RO_LEFT_ARM);
    //readFile(this, "monk_.m", points, triangles, RO_RIGHT_HAND);
    //readFile(this, "monk_.m", points, triangles, RO_LEFT_HAND);
    readFile(this, "monk_rfoot.m", points, triangles, RO_RIGHT_FOOT);
    //readFile(this, "monk_lfoot.m", points, triangles, RO_LEFT_FOOT);
    readFile(this, "monk_rleg.m", points, triangles, RO_RIGHT_LEG);
    readFile(this, "monk_lleg.m", points, triangles, RO_LEFT_LEG);
}

/**
 * Draw the monk.
 * theta:  Rotation angle for animations.
 * omega:  Angle between <0, 0, 1> and the camera projected onto y = 0.  
 * This is for drawing any skill words such that they're facing the direction 
 * of the camera.
 */
void RO_Monk::draw(int theta, float omega)
{
    float barPercent;   // Percent completion for skills
    m.init_materials();

    // Draw the Guillotine Fist characters if the monk casted the skill
    if((castActive || delayActive) && skill == GFIST)
    {
        draw_gfist_chars(omega);
    }

    glPushMatrix();
        // Place the torso in the world
        glTranslatef(x_pos, y_pos, z_pos);
        
        // IF the monk is currently casting a skill
        if(castActive)
        {
            barPercent = (GetTickCount() - initCastTime) 
                / (float)SKILL_TIMES[skill] / castTimePer;

            // Draw the skill timer if under 100 percent
            if(barPercent <= 1.0)
            {
                draw_skill_timer(omega, barPercent);
            }
            // ELSE the skill timer has elapsed; clean up skills
            else
            {
                castActive = false;
                delayActive = true;
                initDelayTime = GetTickCount();

                // Add a sphere
                if(skill == SPHERE)
                {
                    numSpheres++;
                }
                // Go into fury state
                else if(skill == FURY)
                {
                    numSpheres = 0;
                    furyState = true;
                }
                // Reset states and move monk in a direction to simulate
                // the execution of a strike
                else if(skill == GFIST)
                {
                    numSpheres = 0;
                    furyState = false;
                    x_pos -= 3;
                    z_pos += 3;
                }
            }
        }

        // IF the monk is currently incurring a delay
        if(delayActive)
        {
            // Reset the monk if the delay completes
            if(GetTickCount() - initDelayTime >= SKILL_DELAYS[skill])
            {
                delayActive = false;
                x_pos = z_pos = 0;
            }
            // Draw the Guillotine Fist blast during the skill's delay time
            if(skill == GFIST)
            {
                draw_gfist_blast();
            }
        }

        // IF a cast aura is currently surrounding the monk
        if(auraActive)
        {
            // Get the fraction of time the aura has elapsed
            auraFrac = (GetTickCount() - initCastTime) 
                / (float)SKILL_ANIMS[skill];
            glPushMatrix();
                // Draw the monk's aura according to which skill was cast
                if(skill == SPHERE)
                {
                    draw_sphere_cast_aura();
                }
                else if(skill == FURY)
                {
                    draw_fury_cast_aura();
                }
                else if(skill == GFIST)
                {
                    draw_gfist_cast_aura();
                }
            glPopMatrix();
        }

        // IF the monk casted a skill that caused the skill name to show
        if(nameActive)
        {
            draw_skill_name(omega);

            // IF the skill name timer elapses, turn of the name
            if((GetTickCount() - initCastTime) >= TIME_SKILL_NAME)
            {
                nameActive = false;
            }
        }

        // Rotate the model to a more lateral position
        if((auraActive && skill == FURY) || (delayActive && skill == GFIST))
        {
            glRotatef(45, 1.0, 0.0, 0.0);
        }
        glScalef(1.3, 1.3, 1.3);
        glDisable(GL_LIGHTING);

        // Draw the head
        glPushMatrix();
            // Draw the upper headgear
            glPushMatrix();
                draw_upper_headgear();
            glPopMatrix();

            // Draw the middle headgear
            glPushMatrix();
                draw_middle_headgear();
            glPopMatrix();

            // Draw the lower headgear
            glPushMatrix();
                draw_lower_headgear();
            glPopMatrix();

            // Move the head upwards if the body is lowered
            if((auraActive && skill == FURY) 
                || (delayActive && skill == GFIST))
            {
                glTranslatef(0.0, 2.3, 0.3);
                glRotatef(-45, 1.0, 0.0, 0.0);
                glTranslatef(0.0, -2.3, 0.3);
            }
            draw_head();
        glPopMatrix();

        // Draw the left arm
        glPushMatrix();
            // Move the left arm up if the body is lowered
            if((auraActive && skill == FURY) 
                || (delayActive && skill == GFIST))
            {
                glTranslatef(0.0, 1.3, 0.3);
                glRotatef(-45, 1.0, 0.0, 0.0);
                glTranslatef(0.0, -1.3, 0.3);
            }
            // Draw the left hand
            glPushMatrix();
                draw_left_hand();
            glPopMatrix();
            draw_left_arm();
        glPopMatrix();

        // Draw the right arm
        glPushMatrix();
            // Rotate upward if casting a sphere)
            if(castActive && (skill == SPHERE || skill == GFIST))
            {
                glTranslatef(-0.71, 2.49, -0.2);
                glRotatef(-90, 1.0, 0.0, 0.0);
                glTranslatef(0.71, -2.49, 0.2);
            }
            // Draw the right hand
            glPushMatrix();
                draw_right_hand();
            glPopMatrix();
            draw_right_arm();
        glPopMatrix();

        // Draw the left leg
        glPushMatrix();
            // Raise the left leg if the body is lowered
            if((auraActive && skill == FURY) 
                || (delayActive && skill == GFIST))
            {
                glTranslatef(0.0, 0.8, 0.3);
                glRotatef(-45, 1.0, 0.0, 0.0);
                glTranslatef(0.0, -0.8, 0.3);
            }
            // Draw the left foot
            glPushMatrix();
                draw_left_foot();
            glPopMatrix();
            draw_left_leg();
        glPopMatrix();

        // Draw the right leg
        glPushMatrix();
            // Draw the right foot
            glPushMatrix();
                draw_right_foot();
            glPopMatrix();
            draw_right_leg();
        glPopMatrix();
        glEnable(GL_LIGHTING);

        // Draw the spirit spheres
        glPushMatrix();
            // Place the spheres mid-torso
            glTranslatef(0.0, 2.5, 0.0);
            draw_spheres(theta);
        glPopMatrix();

        // Draw the fury state
        glPushMatrix();
            draw_fury_state(theta);
        glPopMatrix();
        glDisable(GL_LIGHTING);

        // Draw the torso
        draw_torso();
    glPopMatrix();
    glEnable(GL_LIGHTING);
}

/**
 * Draw a panda hat on the monk.  Note that for this project, the headgear
 * is fixed; in RO, headgears can be swapped with others.
 */
void RO_Monk::draw_upper_headgear()
{
    glScalef(0.95, 0.95, 0.95);
    drawPart(RO_UPPER_HEADGEAR);
}

/**
 * Draw evil wing ears on the monk.  Note that for this project, the headgear
 * is fixed; in RO, headgears can be swapped with others. (*NOT FEATURED*)
 */
void RO_Monk::draw_middle_headgear()
{
    drawPart(RO_MIDDLE_HEADGEAR);
}

/**
 * Draw a flu mask on the monk.  Note that for this project, the headgear
 * is fixed; in RO, headgears can be swapped with others. (*This function has
 * been co-opted to draw the monk's cape instead.*)
 */
void RO_Monk::draw_lower_headgear()
{
    drawPart(RO_LOWER_HEADGEAR);
}

/**
 * Draw the monk's torso.
 */
void RO_Monk::draw_torso()
{
    m.materials(m.flesh);
    drawPart(RO_TORSO);
}

/**
 * Draw the monk's head.
 */
void RO_Monk::draw_head()
{
    m.materials(m.flesh);
    drawPart(RO_HEAD);
}

/**
 * Draw the monk's right arm.
 */
void RO_Monk::draw_right_arm()
{
    drawPart(RO_RIGHT_ARM);
}

/**
 * Draw the monk's left arm.
 */
void RO_Monk::draw_left_arm()
{
    glTranslatef(0.61, 2.49, -0.2);
    glRotatef(180, 0.0, 1.0, 0.0);
    glTranslatef(0.71, -2.49, 0.2);
    drawPart(RO_RIGHT_ARM);
}

/**
 * Draw the monk's right hand.
 */
void RO_Monk::draw_right_hand()
{
    drawPart(RO_RIGHT_HAND);
}

/**
 * Draw the monk's left hand.
 */
void RO_Monk::draw_left_hand()
{
    drawPart(RO_LEFT_HAND);
}

/**
 * Draw the monk's right foot.
 */
void RO_Monk::draw_right_foot()
{
    drawPart(RO_RIGHT_FOOT);
}

/**
 * Draw the monk's left foot.
 */
void RO_Monk::draw_left_foot()
{
    glTranslatef(0.9, 0.0, 0.0);
    drawPart(RO_RIGHT_FOOT);
}

/**
 * Draw the monk's right leg.
 */
void RO_Monk::draw_right_leg()
{
    drawPart(RO_RIGHT_LEG);
}

/**
 * Draw the monk's left leg.
 */
void RO_Monk::draw_left_leg()
{
    drawPart(RO_LEFT_LEG);
}

/**
 * Draw spirit spheres around the monk.
 * theta:  Rotation angle for the spheres.
 */
void RO_Monk::draw_spheres(int theta)
{
    int sphereNdx; // Current number sphere to draw
    int place = 0; // Current sphere placement angle
    
    glRotatef(theta, 0.0, 1.0, 0.0);
    // FOR each of the monk's active spheres
    for(sphereNdx = 0; sphereNdx < numSpheres; sphereNdx++)
    {
        glPushMatrix();
            place += 72;
            glRotatef(place, 0.0, 1.0, 0.0);
            glTranslatef(3.0, 0.0, 0.0);
            m.materials(m.blue);
            glutSolidSphere(0.3, 100, 100);
        glPopMatrix();
    }
}

/**
 * Draw the monk in fury state. (*NOT FEATURED*)
 */
void RO_Monk::draw_fury_state(int theta)
{
    /* Set the alpha component of the material's diffuse component to something
       less than 1.0 to make it appear transparent.  The first two statements 
       are also required to enable transparency. */
    glEnable(GL_BLEND);
    glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA); 
    
    glRotatef(theta*2, 0.0, 1.0, 0.0);
    
    // IF the monk is in fury state
    if(furyState)
    {

    }
}

/**
 * Draw the skill name above the monk's head.
 * angle:  Display angle.
 */
void RO_Monk::draw_skill_name(float angle)
{
    // Position differential to center the skill name
    float nameCenter = (float)strlen(skillName)*-0.0523;

    // Draw the white skill name lettering
    glPushMatrix();
        // Rotate the letters so they face the camera
        glRotatef(angle, 0.0, 1.0, 0.0);
        // Translate the name to be centered above the monk's head
        glTranslatef(nameCenter, 7.0, 0.01);
        // Scale the lettering down
        glScalef(0.001, 0.001, 0.01);

        // FOR each character in the skill name, draw it
        for(unsigned int i = 0; i < strlen(skillName); i++)
        {
            m.materials(m.white_solid);
            glutStrokeCharacter(GLUT_STROKE_MONO_ROMAN, skillName[i]);
        }
        glutPostRedisplay();
    glPopMatrix();

    // Draw the black bar behind the lettering
    glPushMatrix();
        // Rotate the bar so it's behind the skill lettering
        glRotatef(angle, 0.0, 1.0, 0.0);
        // Draw bar
        glPushMatrix();
            // Translate it behind the words
            glTranslatef(nameCenter - 0.1, 7.05, 0.0);

            // Rotate it so it's facing forward
            glRotatef(90, 0.0, 1.0, 0.0);
            m.materials(m.black);
            gluCylinder(quad, 0.2, 0.2, 2*(-nameCenter + 0.1), 2, 1);
        glPopMatrix();
    glPopMatrix();
}

/**
 * Draw the skill timer above the monk's head.
 * angle:  Display angle.
 * percent:  Percent of the skill timer that is filled.
 */
void RO_Monk::draw_skill_timer(float angle, float percent)
{
    int timerNdx;   // Drawing index for the timer
    glPushMatrix();
        // Rotate the timer so it faces the camera
        glRotatef(angle, 0.0, 1.0, 0.0);
        glPushMatrix();
            glBegin(GL_POLYGON);
            m.materials(m.green);
            // FOR the percent time the skill cast has elapsed
            for(timerNdx = 0; timerNdx < 100*percent; timerNdx++)
            {
                glVertex3f((float)(timerNdx-49)/50, 6.65, 0.0);
                glVertex3f((float)(timerNdx-49)/50, 6.35, 0.0); 
                glVertex3f((float)(timerNdx-50)/50, 6.35, 0.0);
                glVertex3f((float)(timerNdx-50)/50, 6.65, 0.0); 
            }
            glEnd();
        glPopMatrix();
    glPopMatrix();
}

/**
 * Summon a spirit sphere for the monk.
 */
void RO_Monk::summonSpiritSphere()
{
    // IF there are less than 5 active spheres
    if(numSpheres < 5 && !castActive && !delayActive)
    {
        x_pos = z_pos = 0;
        // Increment the number of active spheres 
        initCastTime = GetTickCount();
        castActive = auraActive = nameActive = true;
        skill = SPHERE;
        skillName = SKILL_NAMES[skill];
        if(__DEBUG == 1)
        {
            printf("Summoning spirit sphere #%d at time %ld\n", 
                numSpheres + 1, initCastTime);
        }
    }
    // ELSE print a skill fail message
    else
    {
        printf("Summon Spirit Sphere skill has failed.\n");
    }
}

/**
 * Make the monk go into fury state.
 */
void RO_Monk::fury()
{
    // IF there are exactly 5 active spheres
    if(numSpheres == 5 && !castActive && !delayActive)
    {
        // Reset the number of spheres and go into fury state
        initCastTime = GetTickCount();
        castActive = auraActive = nameActive = true;
        skill = FURY;
        skillName = SKILL_NAMES[skill];
        if(__DEBUG == 1)
        {
            printf("Fury!\n");
        }
    }
    // ELSE print a skill fail message
    else
    {
        printf("Fury skill has failed.\n");
    }
}

/**
 * Make the monk use a guillotine fist.
 */
void RO_Monk::guillotineFist()
{
    // IF there are 5 active spheres and fury state is on
    if(numSpheres == 5 && furyState && !castActive && !delayActive)
    {
        // Reset the number of spheres and go out of fury state
        initCastTime = GetTickCount();
        castActive = auraActive = true;
        skill = GFIST;
        skillName = SKILL_NAMES[skill];
        if(__DEBUG == 1)
        {
            printf("Guillotine Fist!\n");
        }
    }
    // ELSE print a skill fail message
    else
    {
        printf("Guillotine Fist skill has failed.\n");
    }
}

/**
 * Draw an aura that surrounds the monk when Summon Spirit Sphere is cast.
 * The aura can be described as a bright yellow spiky-60-degree-petaled flower
 * whose petal angle decreases slightly as time passes (simulating opening
 * outward slightly).  Its circular base also scales slightly outward as time 
 * passes. The base has an outward sun-like pattern that is separate from the 
 * flower petals.  Within the base there are circular ripples.
 */
void RO_Monk::draw_sphere_cast_aura()
{
    float circleAngle;  // Spoke rotation angle (to simulate a circle)
    float auraAngle;    // Downward angle (to simulate opening)
    float circlePos;    // Distance from center (to simulate opening)
    
    // Enable transparency
    glEnable(GL_BLEND);
    glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA); 

    // Set the material of the aura
    m.materials(m.yellow);

    // FOR each petal
    for(int i = 0; i <= 36; i++)
    {
        // Calculate what angle around the circle it should be positioned at
        circleAngle = (float)360/36*i;
        // Calculate what angle tipped downwards it should be at
        auraAngle = 30 + auraFrac*5;
        // Calculate how far out the petal should be from the center
        circlePos = 2 + auraFrac*0.5;
        glPushMatrix();
            // Rotate it around in a circle according to the angle
            glRotatef(circleAngle, 0, 1, 0);
            // Translate it out according to the aura time
            glTranslatef(circlePos, 0, 0);
            // Draw the sun flare
            glBegin(GL_POLYGON);
            glVertex3f(0.0, 0.0, 1.0);
            glVertex3f(0.0, 0.0, -1.0);
            glVertex3f(1.0, 0.0, 0.0);
            glEnd();
            // Rotate it downwards according to the aura time
            glRotatef(-1*auraAngle, 0, 0, 1);
            // Draw the petal
            glBegin(GL_POLYGON);
            glVertex3f(0.0, 0.0, 1.0);
            glVertex3f(0.0, 0.0, -1.0);
            glVertex3f(0.0, 4.5, 0.0);
            glEnd();
        glPopMatrix();
    }
    // IF auraFrac >= 1, turn off the aura drawing
    if(auraFrac >= 1)
    {
        auraActive = false;
    }
}

/**
 * Draw an aura that surrounds the monk when Fury is cast.  The aura is similar
 * to the Summon Spirit Sphere aura in that the petals start at 60 degrees.  
 * The petals are about as spiky, but the coloring is more white with yellow
 * highlights at the fringes.  As time passes, the petal angle decreases more
 * than the sphere aura, so much that it appears like a plate.  The circular 
 * base also scales outward more.  There are also inner-base ripples, but the 
 * base itself does not have the sun-like pattern.
 */
void RO_Monk::draw_fury_cast_aura()
{
    float circleAngle;  // Spoke rotation angle (to simulate a circle)
    float auraAngle;    // Downward angle (to simulate opening)
    float circlePos;    // Distance from center (to simulate opening)

    // Enable transparency
    glEnable(GL_BLEND);
    glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA); 

    // FOR each petal
    for(int i = 0; i <= 16; i++)
    {
        // Calculate what angle around the circle it should be positioned at
        circleAngle = (float)360/16*i;
        // Calculate what angle tipped downwards it should be at
        auraAngle = 30 + auraFrac*20;
        // Calculate how far out the petal should be from the center
        circlePos = 2 + auraFrac*2;

        // Set the material of the aura
        m.materials(m.yellow);
        glPushMatrix();
            // Rotate it around in a circle according to the angle
            glRotatef(circleAngle, 0, 1, 0);

            // Translate it out according to the aura time
            glTranslatef(circlePos, 0, 0);
            // Rotate it downwards according to the aura time
            glRotatef(-1*auraAngle, 0, 0, 1);
            // Draw the petal
            glBegin(GL_POLYGON);
            glVertex3f(0.0, 0.0, 2.0);
            glVertex3f(0.0, 0.0, -2.0);
            glVertex3f(0.0, 4.0, 0.0);
            glEnd();
        glPopMatrix();

        m.materials(m.white);
        glPushMatrix();
            // Rotate it around in a circle according to the angle
            glRotatef(circleAngle, 0, 1, 0);

            // Translate it out according to the aura time
            glTranslatef(circlePos - 0.1, 0, 0);
            // Rotate it downwards according to the aura time
            glRotatef(-1*auraAngle, 0, 0, 1);
            // Draw the petal
            glBegin(GL_POLYGON);
            glVertex3f(0.0, 0.0, 2.0);
            glVertex3f(0.0, 0.0, -2.0);
            glVertex3f(0.0, 3.0, 0.0);
            glEnd();
        glPopMatrix();
    }

    // IF auraFrac >= 1, turn off the aura drawing
    if(auraFrac >= 1)
    {
        auraActive = false;
    }
}

/**
 * Draw an aura that surrounds the monk when Guillotine is cast.  This aura
 * is completely white and has rigid petals that open in a loop around a small 
 * interval at high angles (about 80 to 70 degrees, and then loop back to 80).
 */
void RO_Monk::draw_gfist_cast_aura()
{
    float circleAngle;  // Spoke rotation angle (to simulate a circle)
    float auraAngle;    // Downward angle (to simulate opening)
    float circlePos;    // Distance from center (to simulate opening)

    // Enable transparency
    glEnable(GL_BLEND);
    glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA); 

    // Set the material of the aura
    m.materials(m.white_solid);

    // FOR each petal
    for(int i = 0; i <= 81; i++)
    {
        // Calculate what angle around the circle it should be positioned at
        circleAngle = (float)360/81*i;
        // Calculate what angle tipped downwards it should be at
        auraAngle = 10 + (int)(auraFrac*50)%10;
        // Calculate how far out the petal should be from the center
        circlePos = 1.5 + auraFrac*.1;
        glPushMatrix();
        // Rotate it around in a circle according to the angle
        glRotatef(circleAngle, 0, 1, 0);
        // Translate it out according to the aura time
        glTranslatef(circlePos, 0, 0);
        // Rotate it downwards according to the aura time
        glRotatef(-1*auraAngle, 0, 0, 1);
        // Draw the petal
        glBegin(GL_POLYGON);
        glVertex3f(0.0, 0.0, 0.2);
        glVertex3f(0.0, 0.0, -0.2);
        glVertex3f(0.0, 4.0, 0.0);
        glEnd();
        glPopMatrix();
    }

    // IF auraFrac >= 1, turn off the aura drawing
    if(auraFrac >= 1)
    {
        auraActive = false;
    }
}

/**
 * Draw the Chinese characters that appear when the monk casts Guillotine Fist.
 * angle:  Angle to display such that the characters face the camera.
 */
void RO_Monk::draw_gfist_chars(float angle)
{
    // Percentage of character texture to draw
    float charPer = (GetTickCount() - initCastTime)/(float)DELAY_GFIST;

    // Don't overdraw the texture
    charPer = charPer >= 1 ? 1 : charPer;

    glPushMatrix();
        glRotatef(angle, 0.0, 1.0, 0.0);
        glPushMatrix();
            glBindTexture(GL_TEXTURE_2D, 1);
            m.materials(m.white_solid);
            glBegin(GL_POLYGON);
            glTexCoord2f(0.0, 0.0);
            glVertex3f(-5.0, 8.0, 0.0); 
            glTexCoord2f(0.0, 1.0);
            glVertex3f(-5.0, 9.0, 0.0);
            glTexCoord2f(charPer, 1.0);
            glVertex3f(-5.0+(10.0*charPer), 9.0, 0.0);
            glTexCoord2f(charPer, 0.0);
            glVertex3f(-5.0+(10.0*charPer), 8.0, 0.0);
            glEnd();
        glPopMatrix();
        glBindTexture(GL_TEXTURE_2D, 0);
    glPopMatrix();
}

/**
 * Draw the explosion that follows the successful connection of a Guillotine 
 * Fist with a target. 
 */
void RO_Monk::draw_gfist_blast()
{
    float waveTime = (GetTickCount() - initDelayTime)/(float)DELAY_GFIST;
    float upPos;
    for(int i = 0; i < 30; i++)
    {
        if(waveTime >= wave[i].time)
        {
            upPos = 100.0*waveTime*(wave[i].time <= 0.2 ? 0.5 : wave[i].time);
            glPushMatrix();
            m.materials(m.light_blue);
            // Rotate around
            glRotatef(wave[i].angle, 0, 1, 0);
            // Rotate down
            glRotatef(-45, 0, 0, 1);
            // Translate up
            glTranslatef(0.0, upPos, 0.0);
            // Rotate up
            glRotatef(-90, 1.0, 0.0, 0.0);
            gluCylinder(quad, 0.1, 0.1, wave[i].length, 2, 1);
            glPopMatrix();
        }
    }
}

